/* Copyright 1998-2011 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#ifndef CUSTOMCURVE_H
#define CUSTOMCURVE_H

#include "stdafx.h"
#include "qubfits.h"

class QUB_Curve_Custom : public IQUB_Curve
{
public:
	QUB_Curve_Custom();
	void __cdecl Free();
	IQUB_Evts * __cdecl Events();
	void __cdecl setExtraVars(char **names, int count);
	const char * __cdecl getName();
	const char * __cdecl getEqn();
	void __cdecl setEqn(const char *x);
	int __cdecl canSetEqn();
	int __cdecl getComponentCount();
	void __cdecl setComponentCount(int x);
	int __cdecl getParamCount();
	const char * __cdecl getParamName(int ix);
	double __cdecl getParamVal(int ix);
	void __cdecl setParamVal(int ix, double val, int sendEvent);
	int __cdecl getParamConst(int ix);
	void __cdecl setParamConst(int ix, int isConst);
	void __cdecl initParams(double *xx, double *yy, int n);
	void __cdecl evalArrayAll(QF_Data *data, double *yy);
	void __cdecl evalArrayOne(QF_Data *data, double *yy, int component);
	int __cdecl hasDeriv();
	void __cdecl evalDerivArray(QF_Data *data, double *dpar);
	const char * __cdecl getMessage();
	QTR_Impl* __cdecl getState();
	void __cdecl updateState(QTR_Impl* state);
private:
	QUB_Evts events;
	QUB_Tree state;
	string message;
	vector<string> extraVars;
	vector<QUB_Tree> stateComponents;
	int ncomp;
	string eqn;
	PyObject* func;
	vector<string> paramNames, fullParamNames;
	vector<double> paramVals;
	vector<int> paramConst;
};




// make your own factory function with this signature,
// returning a new instance of your class:

// vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
IQUB_Curve* __stdcall CreateCustomCurve();
// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

// we init during dll load when the loader should have python GIL
int InitCustomEqn();


#endif
